import 'dart:async';


import 'package:easy_localization/easy_localization.dart';
import 'package:flutter/material.dart';
import 'package:flutter_animated_dialog/flutter_animated_dialog.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import '../../../widget/custom/custom_icon.dart';
import '../../../widget/dialog/custom_dialog_success.dart';
import '../../../widget/dialog/custom_dialog_warning_one_button.dart';
import '../../../widget/dialog/custom_dialog_warning_two_button.dart';
import '../../../widget/pages/page_empty_data_widget.dart';
import '../../../widget/pages/page_error_widget.dart';
import '../../../widget/pages/page_loading_ui_widget.dart';
import '../../../widget/pages/page_loading_widget.dart';
import '../../values/colors.dart';
import 'base_bloc.dart';
import 'base_state.dart';

// ignore: must_be_immutable
class BaseBlocConsumerWidget<B extends BaseBloc> extends StatefulWidget {
  B? bloc;

  final String? messageItemEmpty;
  final bool? isPagingGrid;
  final BlocWidgetBuilder<BaseState>? builder;
  final BlocWidgetListener<BaseState>? listener;
  final BlocBuilderCondition<BaseState>? buildWhen;
  final BlocListenerCondition<BaseState>? listenWhen;
  final List<Type>? listenIf;
  final List<Type>? buildIf;
  final Function()? callBackSignIn;

  BaseBlocConsumerWidget(
      {required this.bloc,
      this.isPagingGrid,
      this.messageItemEmpty,
      this.builder,
      this.listener,
      this.buildWhen,
      this.listenWhen,
      this.listenIf,
      this.buildIf,
      required this.callBackSignIn}) {
    listenIf?.addAll([LoadingUiState]);
    buildIf?.addAll(
        [LoadingState, NoDataState, ServerErrorState, ServerErrorState]);
  }

  @override
  State<BaseBlocConsumerWidget> createState() => _BaseBlocConsumerWidgetState();
}

class _BaseBlocConsumerWidgetState<B extends BaseBloc>
    extends State<BaseBlocConsumerWidget> {
  Timer? _timer;
  bool? isLoading = false;
  bool? isDialog = false;
  B? bloc;

  @override
  void initState() {
    bloc = widget.bloc as B?;
    super.initState();
  }

  @override
  Widget build(BuildContext context,
      {Color color = AppColors.backgroundGrayColor}) {
    if (widget.bloc == null) {
      widget.bloc = BlocProvider.of<B>(context);
    }
    return BlocConsumer<B, BaseState>(
      bloc: bloc,
      listenWhen: (oldState, newState) {
        if (widget.listenWhen != null) {
          return widget.listenWhen!(oldState, newState);
        }
        if (widget.listenIf != null) {
          return widget.listenIf!.any((value) => value == newState.runtimeType);
        }
        return true;
      },
      listener: (context, state) async {
        if (state is LoadingState && isLoading == false) {
          print("|==============> Create LoadingState <==============|");
          timerLoading(context);
          showDialog(
              context: context,
              barrierDismissible: true,
              builder: (BuildContext context) {
                isLoading = true;
                print("|==============> isLoading == true <==============|");
                return LoadingPageWidget();
              }).then((value) {
            print("|==============> isLoading == false --- <==============|");
            isLoading = false;
            _timer?.cancel();
          });
        } else if (state is ServerErrorState) {
          dissmisDialog(context);
          print("|==============> Create ServerErrorState <==============|");
          showAnimatedDialog(
            context: context,
            barrierDismissible: true,
            builder: (BuildContext context) {
              isDialog = true;
              print("|==============> isDialog == true <==============|");
              return CustomDialogOne(
                  content: state.msg,
                  title: "text_warning".tr(),
                  negativeBtnText: "text_ok".tr(),
                  positiveBtnPressed: () {
                    dissmisDialog(context);
                    isDialog = false;
                    print(
                        "|==============> isDialog == false --- <==============|");
                  },
                  colorWarning: AppColors.ErrorColor,
                  iconWarning: CustomIcon(
                    icon: Icons.error,
                    color: Colors.white,
                    size: 24,
                  ));
            },
            animationType: DialogTransitionType.fadeScale,
            curve: Curves.fastOutSlowIn,
            duration: Duration(milliseconds: 100),
          ).then((value) {
            if (state.isReturn == true) {
              dissmisDialog(context);
              isDialog = false;
              print("|==============> isDialog == false --- <==============|");
            }
            return true;
          });
        } else if (state is WarningState) {
          dissmisDialog(context);
          print("|==============> Create WarningState <==============|");
          showAnimatedDialog(
            context: context,
            barrierDismissible: true,
            builder: (BuildContext context) {
              isDialog = true;
              print("|==============> isDialog == true <==============|");
              return CustomDialogOne(
                  content: state.msg,
                  title: "text_warning".tr(),
                  colorWarning: AppColors.WarningColor,
                  negativeBtnText: "text_ok".tr(),
                  iconWarning: CustomIcon(
                    icon: Icons.warning,
                    color: Colors.white,
                    size: 24,
                  ),
                  isClose: false,
                  positiveBtnPressed: () {
                    dissmisDialog(context);
                    isDialog = false;
                    print(
                        "|==============> isDialog == false --- <==============|");
                  });
            },
            animationType: DialogTransitionType.fadeScale,
            curve: Curves.fastOutSlowIn,
            duration: Duration(milliseconds: 100),
          ).then((value) {
            if (state.isReturn == true) {
              dissmisDialog(context);
              isDialog = false;
              print("|==============> isDialog == false <==============|");
            }
            return true;
          });
        } else if (state is UnknowErrorState) {
          dissmisDialog(context);
          print("|==============> Create UnknowErrorState <==============|");
          showAnimatedDialog(
            context: context,
            barrierDismissible: true,
            builder: (BuildContext context) {
              isDialog = true;
              print("|==============> isDialog == true <==============|");
              return CustomDialogTwo(
                content: "${state.code.toString() + " " + state.msg}",
                title: "text_warning".tr(),
                negativeBtnText: "text_cancel".tr(),
                positiveBtnText: "text_accept".tr(),
                positiveBtnPressed: () {
                  isDialog = false;
                  Navigator.of(context).pop();
                },
                negativeBtnPressed: () {
                  isDialog = false;
                  Navigator.of(context).pop();
                },
                iconWarning: CustomIcon(
                  icon: Icons.warning,
                  color: Colors.white,
                  size: 24,
                ),
              );
            },
            animationType: DialogTransitionType.fadeScale,
            curve: Curves.fastOutSlowIn,
            duration: Duration(milliseconds: 100),
          );
        } else if (state is LoginState) {
          // await getIt<LocalDataSource>().resetAllCache();
          dissmisDialog(context);
          print("|==============> Create LoginState <==============|");
          showAnimatedDialog(
            context: context,
            barrierDismissible: true,
            builder: (BuildContext context) {
              isDialog = true;
              print("|==============> isDialog == true <==============|");
              return CustomDialogOne(
                content: state.msg ?? ["text_force_login".tr()],
                title: "text_request_sign_in".tr(),
                isClose: false,
                negativeBtnText: "text_ok".tr(),
                positiveBtnPressed: () {
                  dissmisDialog(context);
                  isDialog = false;
                  widget.callBackSignIn;
                  print(
                      "|==============> isDialog == false --- <==============|");
                },
                iconWarning: CustomIcon(
                  icon: Icons.warning,
                  color: Colors.white,
                  size: 24,
                ),
              );
            },
            animationType: DialogTransitionType.fadeScale,
            curve: Curves.fastOutSlowIn,
            duration: Duration(milliseconds: 100),
          );
        } else if (state is NetwordErrorState) {
          dissmisDialog(context);
          print("|==============> Create NetwordErrorState <==============|");
          showAnimatedDialog(
            context: context,
            barrierDismissible: true,
            builder: (BuildContext context) {
              isDialog = true;
              print("|==============> isDialog == true <==============|");
              return CustomDialogOne(
                  iconWarning: CustomIcon(
                      icon: Icons.signal_wifi_bad, color: AppColors.white),
                  colorWarning: AppColors.WarningColor,
                  content: ["text_time_out".tr()],
                  title: "text_error_connect".tr(),
                  negativeBtnText: "text_ok".tr(),
                  positiveBtnPressed: () {
                    dissmisDialog(context);
                    isDialog = false;
                    print(
                        "|==============> isDialog == false --- <==============|");
                  });
            },
            animationType: DialogTransitionType.fadeScale,
            curve: Curves.fastOutSlowIn,
            duration: Duration(milliseconds: 100),
          );
        } else if (state is SuccessDialogState) {
          dissmisDialog(context);
          print("|==============> Create SuccessDialogState <==============|");
          showAnimatedDialog(
            context: context,
            barrierDismissible: true,
            builder: (BuildContext context) {
              isDialog = true;
              print("|==============> isDialog == true <==============|");
              return CustomDialogSuccess(
                content: state.message,
                title: "text_success".tr(),
                positiveBtnText: 'text_ok'.tr(),
                positiveBtnPressed: () {
                  dissmisDialog(context);
                  isDialog = false;
                  print(
                      "|==============> isDialog == false --- <==============|");
                },
              );
            },
            animationType: DialogTransitionType.fadeScale,
            curve: Curves.fastOutSlowIn,
            duration: Duration(milliseconds: 100),
          ).then((value) {
            if (state.isReturn == true) {
              isDialog = false;
              print("|==============> isDialog == false --- <==============|");
              Navigator.of(context).pop();
            }
            return true;
          });
        }
        widget.listener?.call(context, state);
      },
      buildWhen: (oldState, newState) {
        if (widget.buildWhen != null) {
          return widget.buildWhen!(oldState, newState);
        }
        if (widget.buildIf != null) {
          return widget.buildIf!.any((value) => value == newState.runtimeType);
        }
        return true;
      },
      builder: (context, state) {
        if (widget.builder != null) {
          if (state is LoadingUiState) {
            dissmisDialog(context);
            return LoadingUIPageWidget(
                isCart: false, msg: widget.messageItemEmpty);
          } else if (state is NoDataState) {
            dissmisDialog(context);
            return EmptyUIPageWidget(
              msg: widget.messageItemEmpty ?? "text_no_data".tr(),
              isBackgroundWhite: widget.isPagingGrid,
            );
          } else if (state is ErrorState) {
            dissmisDialog(context);
            return ErrorUIPageWidget(onTap: () {});
          } else {
            return widget.builder!(context, state);
          }
        } else {
          return Container();
        }
      },
    );
  }

  void timerLoading(BuildContext context) {
    print("|==============> Create Timer <==============|");
    _timer?.cancel();
    _timer = Timer(Duration(milliseconds: 2000), () {
      print("|==============> End Timer <==============|");
      if (isDialog == false) {
        dissmisDialog(context);
      }
    });
  }

  void dissmisDialog(BuildContext context) {
    if (isLoading == true) {
      print("|==============> Dissmis Loading **** <==============|");
      _timer?.cancel();
      isLoading = false;
      Navigator.of(context).pop();
    }
    if (isDialog == true) {
      print("|==============> Dismiss Dialog **** <==============|");
      isDialog = false;
      Navigator.of(context).pop();
    }
  }
}
