abstract class BaseState {}

class InitState extends BaseState {}

class LoadingState extends BaseState {
  final isShow;
  LoadingState({this.isShow = false});
}

class LoadingUiState extends BaseState {
  LoadingUiState();
}

class ShowToastState extends BaseState {
  final String msg;
  ShowToastState(this.msg);
}

class OfflineState extends BaseState {}

class NothingState extends BaseState {}

class CallBackState extends BaseState {
  final String? msg;
  CallBackState({this.msg});
}

class WarningBackState extends BaseState {
  final List<String>? msg;
  final String data;
  WarningBackState({required this.data, required this.msg});
}

class NoDataState extends BaseState {
  final String msg;
  NoDataState({required this.msg});
}

class SuccessState<T> extends BaseState {
  final T? data;
  final int? index;
  SuccessState({required this.data, this.index});
}

class SuccessFilterState<T, U> extends BaseState {
  final T? data;
  final U? filter;
  SuccessFilterState({required this.data, required this.filter});
}

class SuccessDialogState extends BaseState {
  final message;
  final bool? isReturn;
  SuccessDialogState({this.message, this.isReturn = false});
}

class FailureState extends BaseState {}

class NetwordErrorState extends BaseState {}

class UnknowErrorState extends BaseState {
  final int code;
  final String msg;
  final String data;
  final bool? isReturn;
  UnknowErrorState(
      {required this.code,
      required this.msg,
      required this.data,
      this.isReturn});
}

class ServerErrorState extends BaseState {
  final int code;
  final List<String>? msg;
  final String data;
  final bool? isReturn;
  ServerErrorState(
      {required this.code,
      required this.msg,
      required this.data,
      this.isReturn});
}

class LoginState extends BaseState {
  final List<String>? msg;
  final String data;
  LoginState({required this.data, required this.msg});
}

class LoginCallBackState extends BaseState {
  final List<String>? msg;
  final String data;
  LoginCallBackState({required this.data, required this.msg});
}

class ErrorState extends BaseState {
  final int code;
  final List<String>? msg;
  final String data;
  final bool? isReturn;
  ErrorState(
      {required this.code,
      required this.msg,
      required this.data,
      this.isReturn});
}

class WarningState extends BaseState {
  final int code;
  final List<String>? msg;
  final String data;
  final bool? isReturn;
  WarningState(
      {required this.code,
      required this.msg,
      required this.data,
      this.isReturn});
}
