import 'dart:ui';
import 'package:auto_route/src/router/auto_router_x.dart';
import '../../values/colors.dart';
import '../../../widget/button/custom_button_back_widget.dart';
import 'package:flutter/material.dart';
import 'package:flutter_bounceable/flutter_bounceable.dart';
import 'package:scroll_to_index/scroll_to_index.dart';

class CustomSliverAppBar extends StatefulWidget {
  const CustomSliverAppBar(
      {Key? key,
      required this.body,
      required this.appBarTitle,
      required this.appBarBackground,
      this.isPrimary = true,
      this.appBarAction,
      required this.controller,
      required this.appBarExpandedHeight,
      required this.appBarCollapsedHeight})
      : super(key: key);

  final Widget body;
  final Widget appBarTitle;
  final Widget appBarBackground;
  final Widget? appBarAction;
  final bool? isPrimary;
  final double? appBarExpandedHeight;
  final double? appBarCollapsedHeight;
  final AutoScrollController? controller;

  @override
  _CustomSliverAppBarState createState() => _CustomSliverAppBarState();
}

class _CustomSliverAppBarState extends State<CustomSliverAppBar> {
  double currentExtent = 0.0;
  double get minExtent => 0.0;
  double get maxExtent => widget.controller?.position.maxScrollExtent ?? 0.0;
  double get deltaExtent => maxExtent - minExtent;
  Curve get curve => Curves.easeOutCubic;

  double actionSpacing = 24;
  double iconStrokeWidth = 1.8;
  double titlePaddingLeftHorizontal = 16;
  double titlePaddingRightHorizontal = 6;
  double opacityBegin = 1.0;
  double opacityEnd = 0;

  Tween<double>? actionSpacingTween = null;
  Tween<double>? iconStrokeWidthTween = null;
  Tween<double>? opacityBeginTween = null;
  Tween<double>? opacityEndTween = null;
  Tween<double>? titlePaddingLeftHorizontalTween = null;
  Tween<double>? titlePaddingRightHorizontalTween = null;

  @override
  void initState() {
    super.initState();
    actionSpacingTween = Tween(begin: 24, end: 0);
    iconStrokeWidthTween = Tween(begin: 1.8, end: 1.2);
    opacityBeginTween = Tween(begin: 1, end: 0);
    opacityEndTween = Tween(begin: 0, end: 1);
    titlePaddingLeftHorizontalTween =
        Tween(begin: 16, end: widget.isPrimary == true ? 10 : 50);
    titlePaddingRightHorizontalTween =
        Tween(begin: 6, end: widget.isPrimary == true ? 36 : 12);
    widget.controller?.addListener(_scrollListener);
  }

  @override
  Widget build(BuildContext context) {
    return NestedScrollView(
      controller: widget.controller,
      physics: AlwaysScrollableScrollPhysics(),
      headerSliverBuilder: (_, __) => [
        SliverAppBar(
          automaticallyImplyLeading: false,
          leading: widget.isPrimary == false
              ? Bounceable(
                  onTap: () => context.popRoute(context),
                  child: CustomButtonBackWiget(opacity: opacityEnd))
              : Container(),
          centerTitle: true,
          actions: [
            Container(
              margin: const EdgeInsets.only(right: 6, left: 6, top: 6),
              child: widget.appBarAction ?? Container(),
            )
          ],
          toolbarHeight: widget.appBarCollapsedHeight!,
          collapsedHeight: widget.appBarCollapsedHeight,
          expandedHeight: widget.appBarExpandedHeight,
          flexibleSpace: FlexibleSpaceCustom(widget.isPrimary),
          backgroundColor: Colors.transparent,
          elevation: 6,
          shadowColor: AppColors.greyShade2,
          pinned: true,
        ),
      ],
      body: widget.body,
    );
  }

  Widget FlexibleSpaceCustom(bool? isPrimary) {
    return isPrimary == true
        ? Container(
            child: Container(
              decoration: BoxDecoration(
                gradient: LinearGradient(
                  begin: Alignment.topCenter,
                  end: Alignment.bottomCenter,
                  colors: [
                    Theme.of(context).primaryColorLight,
                    Theme.of(context).primaryColorDark,
                  ],
                  tileMode: TileMode.repeated,
                ),
              ),
              child: FlexibleSpaceBar.createSettings(
                currentExtent: widget.appBarCollapsedHeight!,
                toolbarOpacity: 1.0,
                child: FlexibleSpaceBar(
                  titlePadding: EdgeInsets.only(
                      left: titlePaddingLeftHorizontal,
                      right: titlePaddingRightHorizontal,
                      bottom: 6),
                  centerTitle: true,
                  title: widget.appBarTitle,
                  background: Opacity(
                      opacity: opacityBegin, child: widget.appBarBackground),
                ),
              ),
            ),
          )
        : Container(
            color: Colors.white,
            child: FlexibleSpaceBar.createSettings(
              currentExtent: widget.appBarCollapsedHeight!,
              toolbarOpacity: 1.0,
              child: FlexibleSpaceBar(
                titlePadding: EdgeInsets.only(
                    left: titlePaddingLeftHorizontal,
                    right: titlePaddingRightHorizontal,
                    bottom: 6),
                centerTitle: true,
                title: widget.appBarTitle,
                background: Opacity(
                    opacity: opacityBegin, child: widget.appBarBackground),
              ),
            ),
          );
  }

  _scrollListener() {
    setState(() {
      currentExtent = widget.controller!.offset;
      actionSpacing = _remapCurrentExtent(actionSpacingTween!);
      iconStrokeWidth = _remapCurrentExtent(iconStrokeWidthTween!);
      titlePaddingLeftHorizontal =
          _remapCurrentExtent(titlePaddingLeftHorizontalTween!);
      titlePaddingRightHorizontal =
          _remapCurrentExtent(titlePaddingRightHorizontalTween!);
      opacityBegin = _remapCurrentExtent(opacityBeginTween!);
      opacityEnd = _remapCurrentExtent(opacityEndTween!);
    });
  }

  double _remapCurrentExtent(Tween<double> target) {
    final double deltaTarget = target.end! - target.begin!;
    double currentTarget =
        (((currentExtent - minExtent) * deltaTarget) / deltaExtent) +
            target.begin!;
    double t = (currentTarget - target.begin!) / deltaTarget;
    double curveT = curve.transform(t);
    return lerpDouble(target.begin!, target.end!, curveT)!;
  }

  @override
  void dispose() {
    widget.controller?.removeListener(_scrollListener);
    widget.controller?.dispose();
    super.dispose();
  }
}
