import 'package:flutter/material.dart';

import '../../common/values/colors.dart';
import '../../common/values/sizes.dart';
import 'custom_icon.dart';

// ignore: must_be_immutable
class CustomProductQualityWidget extends StatefulWidget {
  CustomProductQualityWidget(
      {this.onChange,
      this.quality = 1,
      this.secondaryColor = AppColors.secondaryColor});

  ValueChanged<int>? onChange;
  final int? quality;
  final Color secondaryColor;

  @override
  State<CustomProductQualityWidget> createState() =>
      _CustomProductQualityWidgetState(quality ?? 1);
}

class _CustomProductQualityWidgetState
    extends State<CustomProductQualityWidget> {
  int quality;
  bool isEnableNumberRemote = false;
  final TextEditingController _ctrNumber = TextEditingController();
  final FocusNode _focusNumber = FocusNode();

  _CustomProductQualityWidgetState(this.quality);

  @override
  void initState() {
    super.initState();
    _ctrNumber.text = quality.toString();
  }

  @override
  Widget build(BuildContext context) {
    isEnableNumberRemote = quality > 1 ? true : false;
    return Row(
      mainAxisSize: MainAxisSize.min,
      mainAxisAlignment: MainAxisAlignment.end,
      children: [
        if (isEnableNumberRemote) ...[
          GestureDetector(
            onTap: () {
              if (isEnableNumberRemote && widget.onChange != null) {
                setState(() {
                  if (quality > 1) {
                    quality = quality - 1;
                    _ctrNumber.text = quality.toString();
                    widget.onChange!(quality);
                    if (quality == 1) {
                      isEnableNumberRemote = false;
                    }
                  }
                });
              }
            },
            child: Container(
                width: Sizes.WIDTH_22,
                height: Sizes.WIDTH_22,
                color: Theme.of(context).buttonColor,
                child: CustomIcon(
                  icon: Icons.remove,
                  color: AppColors.white,
                )),
          ),
        ] else ...[
          Container(
              width: Sizes.WIDTH_22,
              height: Sizes.WIDTH_22,
              color: widget.secondaryColor,
              child: CustomIcon(
                icon: Icons.remove,
                color: AppColors.white,
              )),
        ],
        Container(
          width: getWidth(quality),
          decoration: BoxDecoration(
            borderRadius: BorderRadius.circular(10),
            color: AppColors.white,
          ),
          child: TextFormField(
            keyboardType: TextInputType.number,
            decoration: new InputDecoration(
                border: InputBorder.none,
                focusedBorder: InputBorder.none,
                enabledBorder: InputBorder.none,
                errorBorder: InputBorder.none,
                counterText: "",
                disabledBorder: InputBorder.none),
            maxLines: 1,
            maxLength: 10,
            textAlign: TextAlign.center,
            controller: _ctrNumber,
            focusNode: _focusNumber,
            textInputAction: TextInputAction.done,
            onChanged: (value) {
              if (widget.onChange != null && value.length > 0) {
                setState(() {
                  quality = int.parse(value);
                  widget.onChange!(quality);
                });
              } else {
                _ctrNumber.text = quality.toString();
                _ctrNumber.selection = TextSelection.fromPosition(
                    TextPosition(offset: _ctrNumber.text.length));
              }
            },
          ),
        ),
        GestureDetector(
          onTap: () {
            if (widget.onChange != null) {
              setState(() {
                quality = quality + 1;
                _ctrNumber.text = quality.toString();
                widget.onChange!(quality);
                if (quality > 1) {
                  isEnableNumberRemote = true;
                }
              });
            }
          },
          child: Container(
              width: Sizes.WIDTH_22,
              height: Sizes.WIDTH_22,
              color: Theme.of(context).buttonColor,
              child: CustomIcon(
                icon: Icons.add,
                color: Colors.white,
              )),
        )
      ],
    );
  }

  double? getWidth(int quality) {
    double size = 40;
    if (quality > 100) size = 60;
    if (quality > 10000) size = 80;
    if (quality > 1000000) size = 100;
    if (quality > 1000000000) size = 120;
    return size;
  }
}

class AlwaysDisabledFocusNode extends FocusNode {
  @override
  bool get hasFocus => false;
}
