import '../../common/values/colors.dart';
import '../../common/values/sizes.dart';
import '../../common/values/styles.dart';
import '../spaces.dart';
import 'package:easy_localization/src/public_ext.dart';
import 'package:flutter/material.dart';

import '../text/custom_text_medium_bold_field.dart';
import '../text/custom_text_medium_field.dart';

class CustomDialogForgetPassword extends StatelessWidget {
  final String? title;
  final String? account;
  final String? company;
  final String positiveBtnText;
  final GestureTapCallback positiveBtnPressed;

  CustomDialogForgetPassword(
      {required this.positiveBtnText,
      required this.positiveBtnPressed,
      this.title,
      this.account,
      this.company});

  @override
  Widget build(BuildContext context) {
    return Dialog(
      elevation: 0,
      backgroundColor: Colors.transparent,
      child: _buildDialogContent(context),
    );
  }

  Widget _buildDialogContent(BuildContext context) {
    return WillPopScope(
      onWillPop: () async => true,
      child: Container(
        margin: EdgeInsets.only(top: 40),
        decoration: BoxDecoration(
            color: AppColors.white, borderRadius: BorderRadius.circular(8)),
        child: Column(
          mainAxisSize: MainAxisSize.min,
          crossAxisAlignment: CrossAxisAlignment.start,
          children: <Widget>[
            Container(
              padding: EdgeInsets.symmetric(horizontal: 12, vertical: 30),
              child: Column(
                crossAxisAlignment: CrossAxisAlignment.start,
                children: [
                  CustomTextMediumBoldField(
                    label: title,
                    align: TextAlign.left,
                    maxLine: 4,
                    color: AppColors.textColor2,
                  ),
                  SpaceH10(),
                  CustomTextMediumField(
                    label: "text_account".tr() + ": " + account!,
                    align: TextAlign.left,
                    maxLine: 2,
                    color: AppColors.textColor2,
                  ),
                  SpaceH10(),
                  CustomTextMediumField(
                    label: "text_company".tr() + ": " + company!,
                    align: TextAlign.left,
                    maxLine: 2,
                    color: AppColors.textColor2,
                  ),
                ],
              ),
            ),
            Container(
              decoration: BoxDecoration(
                  color: Theme.of(context).primaryColorLight,
                  borderRadius: BorderRadius.only(
                      bottomLeft: Radius.circular(8),
                      bottomRight: Radius.circular(8))),
              width: Sizes.width(context),
              height: 42,
              child: TextButton(
                child: Text(
                  positiveBtnText,
                  style: Styles.customTextMediumStyle(color: Colors.white),
                ),
                onPressed: positiveBtnPressed,
              ),
            ),
          ],
        ),
      ),
    );
  }
}
